<?php

namespace Drupal\mcapi_credcom\CreditCommons;

use Drupal\mcapi\Plugin\Field\FieldType\EntryItem;
use Drupal\mcapi\Entity\Transaction;
use Drupal\mcapi\Entity\Workflow;
use CreditCommons\TransactionInterface;
use CreditCommons\Exceptions\WorkflowViolation;

/**
 * Methods equivalent to the methods on Creditcommons/Workflow
 */
class Workflow {
  protected EntryItem $entry;
  protected string $fromState;
  protected Workflow $data;

  function __construct(Transaction $transaction) {
    $this->entry = $transaction->entries->first();
    $this->fromState = $transaction->state->value;
    $this->data = $transaction->workflow->entity;
  }

  /**
   * Check that the (owner of the) given wallet is permitted to transit the given transaction to the given state.
   *
   * @param string $wallet_id
   * @param string $target_state
   * @return bool
   * @throws CCViolation
   *
   * @todo if admin can do anything is it necessary to call this function in
   * admin mode? or can it not just return true if admin = true?
   */
  public function canTransitionToState(int $wallet_id, string $target_state) {
    if ($this->fromState == TransactionInterface::STATE_INITIATED or $this->fromState == TransactionInterface::STATE_VALIDATED) {
      if ($this->data->direction == 'bill') {
        $permitted_relations = ['payee'];
      }
      elseif ($this->data->direction == 'credit') {
        $permitted_relations = ['payer'];
      }
    }
    else {
      if($this->fromState == $target_state) {
        throw new WorkflowViolation(from: $from_state, to: $target_state, type: $this->data->id);
      }
      $available_states = $this->data->states[$this->fromState];
      if (empty($available_states)) {
        throw new DoesNotExistViolation(type: 'transition', id: "$from_state>>$target_state");
      }
      if (empty($available_states[$target_state])) {
        throw new WorkflowViolation($this->data->id, $this->fromState, $target_state);
      }
      $permitted_relations = $available_states[$target_state]->actors;
    }
    $actual_relation = 'no relation';
    if($wallet_id == $this->entry->payee->id()) {// taken from the first entry.
      $actual_relation = 'payee';
    }
    elseif ($wallet_id == $this->entry->payer->id()) {
      $actual_relation = 'payer';
    }
    return in_array($actual_relation, $permitted_relations);
  }

  /**
   * Determine the relationship(s) of the authenticated user to the current
   * transaction
   * @param $uid
   *   The name of the account, user or wallet, as it appears in the ledger
   * @return array
   *   Whether the given uid is the owner of the payee or payer wallet
   */
  function getMyRelations(int $wid) : array {
    $relations = [];

    return $relations;
  }

}
