<?php

namespace Drupal\mcapi_credcom\Plugin\views\field;
use Drupal\views\ResultRow;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\Core\Form\FormStateInterface;


/**
 * Field handler to display entity label optionally linked to entity page.
 *
 * @ViewsField("credcom_wallet_label")
 */
class WalletLabel extends FieldPluginBase {

  /**
   * {@inheritDoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['link_to_entity'] = ['default' => FALSE];
    return $options;
  }

  /**
   * {@inheritDoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    $form['link_to_entity'] = [
      '#title' => $this->t('Link to wallet'),
      '#description' => $this->t('Link the wallet name to its canonical page.'),
      '#type' => 'checkbox',
      '#default_value' => !empty($this->options['link_to_entity']),
    ];
    parent::buildOptionsForm($form, $form_state);
  }


  /**
   * {@inheritDoc}
   */
  public function render(ResultRow $values) {
    $entity = $values->_entity; // The main entity not the relationship entity.
    //$rel_entity = $this->getEntity($values);
    $name = $this->getValue($values);
    if ($entity instanceOf \Drupal\mcapi\Entity\Transaction) {
      if ($name == \Drupal::config('mcapi_credcom.settings')->get('wallet_name')) {
        $wid = \Drupal::config('mcapi_credcom.settings')->get('wallet_id');
        if ($remote_name = $entity->entries->first()->metadata[$wid]) {
          $parts = explode('/', $remote_name);
          $output = t('@name of @node', ['@name' => array_pop($parts), '@node' => array_pop($parts)]);
          return $this->sanitizeValue($output);
        }
      }
    }

    if (!empty($this->options['link_to_entity'])) {
      try {
        $this->options['alter']['url'] = $wallet->toUrl();
        $this->options['alter']['link_to_entity'] = TRUE;
      }
      catch (UndefinedLinkTemplateException $e) {
        $this->options['alter']['link_to_entity'] = FALSE;
      }
      catch (EntityMalformedException $e) {
        $this->options['alter']['link_to_entity'] = FALSE;
      }
    }
    return $this->sanitizeValue($name);
  }

}

/**
  mc_transactions
  +_entity: Drupal\mcapi\Entity\Transaction {#1271 ▶}
  +_relationship_entities: array:2 [▶]
  +index: 0
  +"mc_transaction_created": "1694602304"
  +"mc_wallet_mc_transaction__entries_1_name": "Balance of Trade" **
  +"mc_transaction__entries_entries_quant": "90"
  +"mc_wallet_mc_transaction__entries_name": "admin"
  +"mc_transaction__entries_entries_description": "test8"
  +"mc_transaction__entries_entries_payee_wid": "1"
  +"xid": "2"
  +"mc_wallet_mc_transaction__entries_wid": "1"
  +"mc_wallet_mc_transaction__entries_1_wid": "3"

  mc_transaction_index
  +_entity: Drupal\mcapi\Entity\Transaction {#1215 ▶}
  +_relationship_entities: array:1 [▶]
  +index: 0
  +"mc_transaction_created": "1694602208"
  +"mc_transaction__entries_entries_description": "test8"
  +"mc_wallet_mc_transaction__entries_name": "Balance of Trade"
  +"mc_transaction__entries_entries_quant": "60"
  +"xid": "1"
  +"mc_wallet_mc_transaction__entries_wid": "3"
 */