<?php

namespace Drupal\mcapi_forms;
use Drupal\mcapi_forms\Entity\McForm;
use Drupal\mcapi\Form\TransactionForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\mcapi\Plugin\Field\FieldWidget\EntryWidget;
use Drupal\Core\Routing\RouteMatchInterface;

/**
 * Form builder for transaction formed designed via the UI.
 */
class DesignedTransactionForm extends TransactionForm {
  protected $config;

  /**
   * {@inheritDoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\Core\Entity\Entity\EntityFormDisplay $display */
    $display = $form_state->get('form_display');
    $components = $display->getComponents();
    foreach($components as $name => $info) {
      $display->removeComponent($name);
    }
    $form['#title'] = $this->config->label();
    $form['#twig_template'] = str_replace(["\r\n","\n","\r"], "<br/>", $this->config->getTwigTemplate());
    $form = parent::form($form, $form_state);
    // Move all the tokens to the top level of the form for processing.
    foreach (EntryWidget::rawElements($this->entity->entries, 0) as $id => $element)  {
      $tok = substr($id, -4) == '_wid' ? substr($id, 0, -4) : $id;
      if (preg_match('/\{\{ ?'.$tok.' ?\}\}/', $form['#twig_template'])) {
        $form['#tokens'][$id] = $tok;
        $form[$id] = $element;
      }
    }
    $added_fields = array_filter(
      \Drupal::service('entity_field.manager')->getFieldDefinitions('mc_transaction', 'mc_transaction'),
      function ($f) {return !$f instanceOf \Drupal\Core\Field\BaseFieldDefinition;}
    );
    foreach ($added_fields as $field_definition) {
      $fname = $field_definition->getName();
      if (preg_match('/\{\{ ?'.$fname.' ?\}\}/', $form['#twig_template'])) {
        $form['#tokens'][$fname] = $fname;
        $display->setComponent($fname, $components[$fname]);
        $items = $this->getEntity()->get($fname);
        $form[$fname] = $display->getRenderer($fname)->form($items, $form, $form_state)['widget'];
      }
    }
    $form['#twig_template'] = str_replace(["\r\n","\n","\r"], "<br/>", $this->config->getTwigTemplate());
    $form['#attached']['library'][] = 'mcapi_forms/display';
    $form['#after_build'][] = '::prepareTemplateForm';
    return $form;
  }

  /**
   * {@inheritDoc}
   */
  protected function actions(array $form, FormStateInterface $form_state) {
    $actions = parent::actions($form, $form_state);
    $actions['submit']['#value'] = $this->t('Send');
    // @todo check this is working.
    $form['#cache']['contexts'][] = 'user';
    return $actions;
  }

  /**
   * {@inheritDoc}
   */
  public function getFormId() {
    return 'designed_transaction_form';
  }


  /**
   * Provide an ID for the transaction form consistent with the keys of hook_transction_form_list().
   */
  function transactionFormId() {
    return 'designed_'.$this->config->id();
  }

  /**
   * {@inheritDoc}
   */
  public function buildEntity(array $form, FormStateInterface $form_state) {
    $form_state->setValue('entries', [
      'description' => $form_state->getValue('description'),
      'quant' => $form_state->getValue('quant'),
      'payee_wid' => $form_state->getValue('payee_wid'),
      'payer_wid' => $form_state->getValue('payer_wid'),
      'primary' => TRUE
    ]);
    return parent::buildEntity($form, $form_state);
  }

  /**
   * Form #afterbuild callback
   * Break up the template into render elements alternating between markup and form elements.
   */
  function prepareTemplateForm($form, $form_state) {
    foreach ($form['#tokens'] as $fname => $tok) {
      $element = $form[$fname];
      unset($form[$fname]);
      if (isset($element['widget'][0])) {// Additional fields
        $element = $element['widget'][0]['value'];
      }
      $replacements[$tok] = $element;
    }
    $w = 0;
    $tok = strtok(' '.$form['#twig_template'], '{');// ensure the first is text
    while ($tok !== FALSE) {
      $parts[] = $tok;
      $tok = strtok('{}');
    }
    foreach ($parts as $key => $text) {
      if ($key % 2 == 0) { //even numbers are text
        $form[] = [
          '#markup' => $text,
          '#weight' => $w++
        ];
      }
      elseif (isset($replacements[trim($text)])) { // odd numbers are tokens
        $element = $replacements[trim($text)];
        unset($element['#theme_wrappers']);
        $form[] = ['#weight' => $w++] + $element;
      }
      else {
        throw new \Exception("Unrecognised token in designed transaction form: $text");
      }
    }
    return $form;
  }


  /**
   * {@inheritDoc}
   */
  public function getEntityFromRouteMatch(RouteMatchInterface $route_match, $entity_type_id) {
    $route_name = $route_match->getRouteName();
    $mc_form_id = substr($route_name, strrpos($route_name, '.') + 1);
    $this->config = McForm::load($mc_form_id);
    $this->request->attributes->set('mc_workflow', $this->config->getWorkflowId());
    return parent::getEntityFromRouteMatch($route_match, $entity_type_id);
  }


}
