<?php

namespace Drupal\mcapi_forms\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\mcapi\Entity\Workflow;

/**
 * Defines the designed form entity class.
 *
 * @ConfigEntityType(
 *   id = "mc_form",
 *   label = @Translation("Transaction form"),
 *   config_prefix = "form",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *   },
 *   handlers = {
 *     "form" = {
 *       "edit" = "Drupal\mcapi_forms\FormEditorForm",
 *       "add" = "Drupal\mcapi_forms\FormEditorForm"
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\mcapi_forms\Entity\McFormRouteProvider",
 *     },
 *   },
 *   links = {
 *     "add-form" = "/admin/accounting/form/create",
 *     "edit-form" = "/admin/accounting/form/edit/{mc_form}",
 *   },
 *   admin_permission = "configure mcapi",
 *   config_export = {
 *     "id",
 *     "label",
 *     "wallet_link_title",
 *     "workflow_type",
 *     "path",
 *     "twig",
 *     "menu",
 *     "add_transaction_tab",
 *     "quicklink"
 *   }
 * )
 */
class McForm extends ConfigEntityBase {

  /**
   * Unique ID
   * @var string
   */
  protected $id;

  /**
   * Name to show in lists
   * @var string
   */
  protected $label;

  /**
   * Short description
   * @var string
   */
  protected $wallet_link_title;

  /**
   * Whether this workflow is allowed to be used
   * @var bool
   */
  protected $workflow_type;

  /**
   * Whether this is BILL or CREDIT or THIRDPARTY
   * @var string
   */
  protected $path;

  /**
   * Structured data
   * @var array
   */
  protected $twig;

  /**
   * The menu link title, parent menu, and weight
   * @var array
   */
  public $menu;


  /**
   * TRUE to show a link to this form in the wallet extrafields quicklinks.
   * @var bool
   */
  protected $quicklink;

  function getWalletLinkTitle() : string {
    return (string) $this->wallet_link_title;
  }

  function getWorkflowId() : string {
    return (string) $this->workflow_type;
  }
  function setWorkflowId($workflow_id) : string {
    $this->workflow_type = $workflow_id;
    return $this;
  }
  function setWorkflow(string $wf) : static {
    $this->workflow_type = $wf;
    return $this;
  }

  function getWorkflow() : Type {
    return Workflow::load($this->workflow_type);
  }

  function getPath() : string {
    return (string) $this->path;
  }

  function getMenuItem() : \stdClass {
    return (object) $this->menu;
  }

  function hasTransactionTab() : bool {
    return (bool) $this->menu['add_transaction_tab'];
  }

  function getTwigTemplate() : string {
    return (string) $this->twig;
  }

  function setTwigTemplate(string $twig) : static {
    $this->twig = $twig;
    return $this;
  }

 function getQuicklink() : bool {
   return (bool)$this->quicklink;
 }

}
