<?php

namespace Drupal\mcapi_limits\Plugin\Field\FieldType;

use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataReferenceTargetDefinition;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;

/**
 * Plugin implementation of the 'worth' field type.
 *
 * @FieldType(
 *   id = "mc_wallet_limits",
 *   label = @Translation("Limits"),
 *   category = @Translation("Community Accounting"),
 *   description = @Translation("Min and max balance limits"),
 *   default_formatter = "mcapi_limits",
 *   default_widget = "mcapi_limits",
 *   list_class = "\Drupal\mcapi_limits\Plugin\Field\LimitsFieldItemList"
 * )
 *
 * @todo make a validator for this field.
 */
class WalletLimitsItem extends FieldItemBase {

  /**
   * {@inheritDoc}
   */
  public static function propertydefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['min'] = DataReferenceTargetDefinition::create('integer')
      ->setLabel(new TranslatableMarkup('Minimum'))
      ->setRequired(TRUE);
    $properties['max'] = DataReferenceTargetDefinition::create('integer')
      ->setLabel(new TranslatableMarkup('Maximum'))
      ->setRequired(FALSE);
    return $properties;
  }

  /**
   * {@inheritDoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    return [
      'columns' => [
        'min' => [
          'description' => 'minimum limit',
          'type' => 'int',
          'size' => 'normal',
          'not null' => FALSE,
        ],
        'max' => [
          'description' => 'maximum limit',
          'type' => 'int',
          'size' => 'normal',
          'not null' => FALSE,
        ]
      ]
    ];
  }

  /**
   * {@inheritDoc}
   */
  public function isEmpty() {
    return is_null($this->min) and is_null($this->max);
  }

  /**
   * {@inheritDoc}
   */
  public static function generateSampleValue(FieldDefinitionInterface $field_definition) {
    $wids = \Drupal::entityQuery('mc_wallet')->accessCheck(TRUE)->execute();
    shuffle($wids);
    return [
      'min' => -1000,
      'max' => 1000
    ];
  }

  function getValue() {
    return [
      'min' => $this->min,
      'max' => $this->max
    ];
  }

  /**
   * {@inheritDoc}
   */
  public static function mainPropertyName() {
    return 'min';
  }

  public function fieldSettingsForm(array $form, FormStateInterface $form_state) {
    return [
      '#markup' => t(
        'Edit the default balance limits <a href=":link">here</a>',
        [':link' => Url::fromRoute('mcapi_limits.settings_form')->toString()]
      )
    ];
  }
}
