<?php

namespace Drupal\mcapi_limits\Plugin\Validation\Constraint;

use Drupal\mcapi\Functions;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Constraint;

/**
 * Entity level constraint validator checking whether a wallet is beyond its limits.
 */
class TransactionLimitsConstraintValidator extends ConstraintValidator {

  /**
   * {@inheritdoc}
   */
  public function validate($transaction, Constraint $constraint) {
    // Curiously this runs even if the field level validation fails.
    // Skip validation if a wallet id is invalid.
    if ($transaction->entries->first()->payee and $transaction->entries->first()->payer) {
      if ($transgressions = mcapi_limits_check($transaction)) {
        foreach ($transgressions as $wid => $transgression) {
          $replacements = [
            '%wallet' => $transgression->wallet->label(),
            '%limit' => Functions::formatCurrency($transgression->limit, FALSE),
            '%excess' => Functions::formatCurrency($transgression->diff, FALSE)
          ];
          $this->context
            ->buildViolation($constraint->message, $replacements)
            ->atPath('entries.0.quant')
            ->addViolation();
        }
      }
    }
  }

}