<?php

/**
 * @file
 */
namespace Drupal\mcapi_limits\Plugin\migrate\process;

use Drupal\mcapi\Entity\Storage\WalletStorage;
use Drupal\migrate\MigrateExecutableInterface;
use Drupal\migrate\ProcessPluginBase;
use Drupal\migrate\Row;
use Drupal\migrate\MigrateSkipRowException;
use Drupal\user\Entity\User;
use Drupal\mcapi\EventSubscriber\MigrationSubscriber;

/**
 * convert the old display setting into the new one-string format
 *
 * @MigrateProcessPlugin(
 *   id = "d7_mc_wallet_limit"
 * )
 */
class WalletLimits extends ProcessPluginBase {

  /**
   * {@inheritDoc}
   *
   * $user->data['limits_personal'][$currcode]['min'] = $min;
   * We convert it to go in its own table with fields
   * ['wid', 'curr_id', 'max', 'value', 'editor', 'date']
   */
  public function transform($value, MigrateExecutableInterface $migrate_executable, Row $row, $destination_property) {
    //Incoming value is nothing.
    //would be the $user->data array from D7, except it doesn't come through because it is an array
    $value = $row->getSourceProperty('data');
    $uid = $row->getSourceProperty('uid');
    if (!$uid) {
      throw new MigrateSkipRowException('no uid source property');
    }
    $user = User::load($uid);
    if (!$user) {
      throw new MigrateSkipRowException('User not migrated');
    }

    $currcode = MigrationSubscriber::getMostUsedCurrency($this);
    if (isset($value['limits_personal'][$currcode])) {
      $limits = $value['limits_personal'][$currcode];
      foreach (WalletStorage::walletsOf($user) as $wid) {
        // Apply the same limits to every wallet held by the user
        return [
          'wid' => $wid,
          'min' => $limits['min'],
          'max' => $limits['max'],
        ];
      }
    }
    throw new MigrateSkipRowException("No limits to migrate");
  }

}
