<?php

namespace Drupal\mcapi_signatures\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;

/**
 * Defines the wallet entity.
 *
 * The metaphor is not perfect because this wallet can hold a negaive balance.
 * A more accurate term, 'account', has another meaning in Drupal.
 *
 * This Entity knows all about the special intertrading wallets though none are
 * installed by default.
 *
 * @note Wallet 'holders' could be any content entity, but the wallet owner is
 * always a user, typically the owner of the holder entity
 *
 * @ContentEntityType(
 *   id = "mcapi_wallet",
 *   label = @Translation("Wallet DO NOT USE"),
 *   module = "mcapi_signatures",
 *   handlers = {
 *     "views_data" = "Drupal\mcapi\Entity\Views\WalletViewsData"
 *   },
 *   base_table = "mcapi_wallet",
 *   entity_keys = {
 *     "label" = "name",
 *     "id" = "wid",
 *     "uuid" = "uuid"
 *   }
 * )
 */
class Wallet extends ContentEntityBase {

  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);
    $fields['holder'] = BaseFieldDefinition::create('dynamic_entity_reference')
      ->setLabel(t('Holder (dummy field)'))
      ->setSettings([
        'user' => [
          'handler' => 'default:user',
          'autowallet' => '1'
        ]
      ]);
    return $fields;
  }
}
