<?php

namespace Drupal\mcapi\Entity\Storage;

use Drupal\mcapi\Entity\Wallet;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\Sql\SqlContentEntityStorage;

/**
 * Storage controller for wallet entity.
 */
class WalletStorage extends SqlContentEntityStorage implements WalletStorageInterface {

  /**
   * {@inheritDoc}
   */
  public function getQueryServiceName() {
    return 'mc_wallet.query.sql';
  }

  /**
   * {@inheritDoc}
   */
  public static function walletsOf(ContentEntityInterface $entity, $load = FALSE) {
    $wids = \Drupal::entityQuery('mc_wallet')
      ->accessCheck(FALSE) // not to trigger hook_query_mc_wallet_access_alter()
      ->condition('holder.target_type', $entity->getEntityTypeId())
      ->condition('holder.target_id', $entity->id())
      ->execute();
    return $load ?
      Wallet::loadMultiple($wids) :
      $wids;
  }

  /**
   * Return the first wallet held by a given entity.
   *
   * @param ContentEntityBase $entity
   *   Defaults to the current user.
   *
   * @return int
   *   0 if no wallets are held
   */
  static function firstWalletOfEntity(ContentEntityInterface $entity) : int {
    if ($wids = self::walletsOf($entity)) {
      return reset($wids);
    }
    return 0;
  }

  static function hasWallet(ContentEntityInterface $entity) : bool {
    return (bool)static::walletsOf($entity, FALSE);
  }

}

