<?php

namespace Drupal\mcapi\Entity\ViewBuilder;

use Drupal\Core\Entity\EntityViewBuilder;

/**
 * Render controller for wallets.
 */
class WalletViewBuilder extends EntityViewBuilder {

  /**
   * {@inheritDoc}
   */
  public function buildComponents(&$build, array $entities, array $displays, $view_mode) {
    $display = $displays['mc_wallet'];
    foreach ($entities as $id => $wallet) {
      if ($view_mode <> 'full') {
        // Show the wallet title at the top
        $build[$id]['title'] = [
          '#type' => 'item',
          '#markup'=> '<strong>'. $wallet->label(). '('. $wallet->id() .')</strong>',
          '#weight' => -2,
        ];
      }
      if ($options = $display->getComponent('balance_bars')) {
        $build[$id]['balance_bars'] = mc_wallet_view_balance_bars($wallet) + [
          '#title' => $this->t('Given and received'),
          '#weight' => $options['weight'],
          '#theme_wrappers' => ['mc_wallet_component']
        ];
      }
      if ($options = $display->getComponent('balance_viz')) {
        $build[$id]['balance_viz'] = mc_wallet_view_balance_viz($wallet) + [
          '#title' => $this->t('Balance'),
          '#weight' => $options['weight'],
          '#theme_wrappers' => ['mc_wallet_component']
        ];
      }
      if ($options = $display->getComponent('stats')) {
        $build[$id]['stats'] = mc_wallet_view_stats($wallet) + [
          '#title' => $this->t('Summary stats'),
          '#weight' => $options['weight'],
          '#theme_wrappers' => ['mc_wallet_component']
        ];
      }
      if ($options = $display->getComponent('histories')) {
        $build[$id]['histories'] = mc_wallet_view_balance_history($wallet, 300, 150) + [
          '#title' => $this->t('Balance history'),
          '#weight' => $options['weight'],
          '#theme_wrappers' => ['mc_wallet_component']
        ];
      }
      if ($view_mode <> 'full' and $view_mode <> 'default') {
        // Also see Drupal\mcapi\Plugin\Block\HeldWallets
        $build[$id]['links'] = mc_wallet_links($wallet);
      }
    }
  }

}
