<?php

namespace Drupal\mcapi\Entity;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\user\UserInterface;

/**
 * Provides an interface defining an exchange entity.
 *
 * @note This does NOT implement entityOwnerInterface because the owner is derived
 * from the holder, so you can't set the owner
 */
interface WalletInterface extends ContentEntityInterface {

  /**
   * Get the trading stats for all currencies allowed or used in the wallet.
   *
   * @param bool $include_empty
   *   Return empty stats for unused currencies.
   *
   * @return array
   *   Arrays of statistics
   *
   * @see \Drupal\mcapi\Entity\Storage\TransactionStorageInterface::summaryData()
   */
  public function getAllStats() : \stdClass;

  /**
   * Get the standard trading stats.
   *
   * @return array
   *   Array of statistics, keyed by stat name.
   *
   * @see $this->getStat()
   */
  public function getStat(string $name) : int;

  /**
   * Returns the entity owner's user entity.
   *
   * @return \Drupal\user\UserInterface
   *   The owner user entity.
   */
  public function getOwner() : UserInterface;

  /**
   * Returns the entity owner's user ID.
   *
   * @return int|null
   *   The owner user ID, or NULL in case the user ID field has not been set on
   *   the entity.
   */
  public function getOwnerId() : int;

  /**
   * Check whether a wallet has ever been used. i.e. whether the journal
   * references it.
   *
   * @return bool
   *   TRUE if the wallet has never been used.
   */
  public function isEmpty() : bool;

}
