<?php

namespace Drupal\mcapi\Plugin\Block;

use Drupal\mcapi\Functions;
use Drupal\mcapi\Plugin\Block\WalletContextBlockBase;
use Drupal\user\Entity\User;
use Drupal\user\UserInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\Markup;

/**
 * Provides a block for switching users.
 *
 * @Block(
 *   id = "mcapi_pending",
 *   admin_label = @Translation("Users' pending transactions"),
 *   category = "Community accounting"
 * )
 */
class Pending extends WalletContextBlockBase implements ContainerFactoryPluginInterface {

  /**
   * {@inheritDoc}
   */
  public function label() {
    if ($this->configuration['holder_source'] == static::HOLDER_CURRENT_USER) {
      $label = $this->t('Pending transactions');
    }
    else {
      // only the settings page should have no holder.
      $owner = isset($this->holder) ? $this->getWalletOwner()->getDisplayName() : '[user:display-name]';
      $label = $this->t('Pending transactions for @name', ['@name' => $owner]);
    }
    return $label;
  }

  /**
   * {@inheritDoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
      'empty_message' => $this->t('No pending transactions.'),
      'show_other' => TRUE
    ];
  }

  /**
   * {@inheritDoc}
   */
  public function blockForm($form, $form_state) {
    $form = parent::blockForm($form, $form_state);
    $form['empty_message'] = [
      '#title' => $this->t('Empty message'),
      '#description' => $this->t('leave blank to hide the block when there are no pending transactions'),
      '#type' => 'textfield',
      '#default_value' => $this->configuration['empty_message'],
      '#placeholder' => $this->t('No pending transactions.')
    ];
    $form['show_other'] = [
      '#title' => $this->t('Show all'),
      '#description' => t("Include also pending transactions awaiting others' signatures"),
      '#type' => 'checkbox',
      '#default_value' => $this->configuration['show_other'],
    ];
    return $form;
  }

  /**
   * Get the owner i.e. the user responsible for the wallet held
   */
  private function getWalletOwner() : User {
    if ($this->holder instanceOf UserInterface) {
      $owner = $this->holder;
    }
    else {
      $owner = $this->holder->getOwner();
    }
    return $owner;
  }

  /**
   * {@inheritDoc}
   */
  public function build(): array {
    $renderable = parent::build();
    $uid = $this->getWalletOwner()->id();
    if ($this->configuration['show_other']) {
      $xids = Functions::transactionsNeedingSig($uid);
    }
    else {
      $xids = Functions::transactionsNeedingSig($uid);
    }
    if ($xids) {
      $renderable += mcapi_render_sentences($xids, TRUE);
    }
    elseif ($this->configuration['empty_message']) {
      // @todo put an optional empty message
      $renderable[] = [
        '#markup' => Markup::create($this->configuration['empty_message']),
        '#weight' => -1
      ];
    }
    return $renderable;
  }

}

