<?php

namespace Drupal\mcapi\Plugin\Validation\Constraint;

use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Constraint;

/**
 * Check that the two wallets in the transaction are different.
 */
class DifferentWalletsConstraintValidator extends ConstraintValidator {

  /**
   * {@inheritDoc}
   */
  public function validate($entries, Constraint $constraint) {
    // Check the payer and payee aren't the same.
    foreach ($entries as $delta => $entry) {
      if (is_null($entry->payee)) {
        $this->context->addViolation($constraint->missingWallet, ['@id' => $entry->payee_wid]);
      }
      if (is_null($entry->payer)) {
        $this->context->addViolation($constraint->missingWallet, ['@id' => $entry->payer_wid]);
      }
      if ($entry->primary and $entry->payer_wid == $entry->payee_wid) {
        $this->context
          ->buildViolation($constraint->sameMessage)
          ->atPath("$delta.payee_wid")
          ->addViolation();
        $this->context
          ->buildViolation($constraint->sameMessage)
          ->atPath("$delta.payer_wid")
          ->addViolation();
      }
    }
  }

}
